import argparse
import json

from nubia_score import Nubia


def main(args):
    metric = Nubia()
    six_dim = args.six_dim.lower() == "true"

    with open(args.output_file, "w") as out:
        with open(args.input_file, "r") as f:
            for line in f:
                data = json.loads(line)
                candidate = data["candidate"]
                reference = data["reference"]

                scores = metric.score(
                    reference,
                    candidate,
                    get_features=True,
                    six_dim=six_dim,
                    aggregator=args.aggregator,
                )
                out.write(json.dumps({"nubia": scores}) + "\n")


if __name__ == "__main__":
    argp = argparse.ArgumentParser()
    argp.add_argument("--input-file", required=True)
    argp.add_argument("--six-dim", required=True)
    argp.add_argument("--aggregator", required=True)
    argp.add_argument("--output-file", required=True)
    args = argp.parse_args()
    main(args)



# Copyright (c) 2020, Salesforce.com, Inc.
# Copyright 2018 The Google AI Language Team Authors and The HuggingFace Inc. team.
# Copyright (c) 2018, NVIDIA CORPORATION.  All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
""" Finetuning the library models for sequence classification on GLUE (Bert, XLM, XLNet)."""
# fmt: off
from __future__ import absolute_import, division, print_function

"""
This code was modified from https://github.com/salesforce/factCC/blob/40a8a5684ec7131bfc92396d9b7f236f80095665/modeling/run.py
to allow for outputting predictions to a file.
"""

import argparse
import glob
import json
import logging
import os
import random

import wandb
import numpy as np
import torch

from model import BertPointer
from utils import (compute_metrics, convert_examples_to_features, output_modes, processors)

from torch.utils.data import (DataLoader, RandomSampler, SequentialSampler, TensorDataset)
from torch.utils.data.distributed import DistributedSampler
from tqdm import tqdm, trange
from pytorch_transformers import (WEIGHTS_NAME, BertConfig, BertForSequenceClassification, BertTokenizer)

from pytorch_transformers import AdamW, WarmupLinearSchedule

logger = logging.getLogger(__name__)
wandb.init(project="entailment-metric")

ALL_MODELS = sum((tuple(conf.pretrained_config_archive_map.keys()) for conf in (BertConfig,)), ())

MODEL_CLASSES = {
    'pbert': (BertConfig, BertPointer, BertTokenizer),
    'bert': (BertConfig, BertForSequenceClassification, BertTokenizer),
}


def set_seed(args):
    random.seed(args.seed)
    np.random.seed(args.seed)
    torch.manual_seed(args.seed)
    if args.n_gpu > 0:
        torch.cuda.manual_seed_all(args.seed)


def make_model_input(args, batch):
    inputs = {'input_ids':        batch[0],
              'attention_mask':   batch[1],
              'token_type_ids':   batch[2],
              'labels':           batch[3]}

    # add extraction and augmentation spans for PointerBert model
    if args.model_type == "pbert":
        inputs["ext_mask"] = batch[4]
        inputs["ext_start_labels"] = batch[5]
        inputs["ext_end_labels"] = batch[6]
        inputs["aug_mask"] = batch[7]
        inputs["aug_start_labels"] = batch[8]
        inputs["aug_end_labels"] = batch[9]
        inputs["loss_lambda"] = args.loss_lambda

    return inputs


def train(args, train_dataset, model, tokenizer):
    """ Train the model """

    args.train_batch_size = args.per_gpu_train_batch_size * max(1, args.n_gpu)
    train_sampler = RandomSampler(train_dataset) if args.local_rank == -1 else DistributedSampler(train_dataset)
    train_dataloader = DataLoader(train_dataset, sampler=train_sampler, batch_size=args.train_batch_size)

    if args.max_steps > 0:
        t_total = args.max_steps
        args.num_train_epochs = args.max_steps // (len(train_dataloader) // args.gradient_accumulation_steps) + 1
    else:
        t_total = len(train_dataloader) // args.gradient_accumulation_steps * args.num_train_epochs

    # Prepare optimizer and schedule (linear warmup and decay)
    no_decay = ['bias', 'LayerNorm.weight']
    optimizer_grouped_parameters = [
        {'params': [p for n, p in model.named_parameters() if not any(nd in n for nd in no_decay)], 'weight_decay': args.weight_decay},
        {'params': [p for n, p in model.named_parameters() if any(nd in n for nd in no_decay)], 'weight_decay': 0.0}
        ]
    optimizer = AdamW(optimizer_grouped_parameters, lr=args.learning_rate, eps=args.adam_epsilon)
    scheduler = WarmupLinearSchedule(optimizer, warmup_steps=args.warmup_steps, t_total=t_total)
    if args.fp16:
        try:
            from apex import amp
        except ImportError:
            raise ImportError("Please install apex from https://www.github.com/nvidia/apex to use fp16 training.")
        model, optimizer = amp.initialize(model, optimizer, opt_level=args.fp16_opt_level)

    # multi-gpu training (should be after apex fp16 initialization)
    if args.n_gpu > 1:
        model = torch.nn.DataParallel(model)

    # Distributed training (should be after apex fp16 initialization)
    if args.local_rank != -1:
        model = torch.nn.parallel.DistributedDataParallel(model, device_ids=[args.local_rank],
                                                          output_device=args.local_rank,
                                                          find_unused_parameters=True)

    # Train!
    logger.info("***** Running training *****")
    logger.info("  Num examples = %d", len(train_dataset))
    logger.info("  Num Epochs = %d", args.num_train_epochs)
    logger.info("  Instantaneous batch size per GPU = %d", args.per_gpu_train_batch_size)
    logger.info("  Total train batch size (w. parallel, distributed & accumulation) = %d",
                args.train_batch_size * args.gradient_accumulation_steps * (torch.distributed.get_world_size() if args.local_rank != -1 else 1))
    logger.info("  Gradient Accumulation steps = %d", args.gradient_accumulation_steps)
    logger.info("  Total optimization steps = %d", t_total)

    global_step = 0
    tr_loss, logging_loss = 0.0, 0.0
    model.zero_grad()
    train_iterator = trange(int(args.num_train_epochs), desc="Epoch", disable=args.local_rank not in [-1, 0])
    set_seed(args)  # Added here for reproductibility (even between python 2 and 3)

    for epoch_ix in train_iterator:
        epoch_iterator = tqdm(train_dataloader, desc="Iteration", disable=args.local_rank not in [-1, 0])
        for step, batch in enumerate(epoch_iterator):
            model.train()
            batch = tuple(t.to(args.device) for t in batch)
            inputs = make_model_input(args, batch)
            outputs = model(**inputs)
            loss = outputs[0]  # model outputs are always tuple in pytorch-transformers (see doc)

            if args.n_gpu > 1:
                loss = loss.mean() # mean() to average on multi-gpu parallel training
            if args.gradient_accumulation_steps > 1:
                loss = loss / args.gradient_accumulation_steps

            if args.fp16:
                with amp.scale_loss(loss, optimizer) as scaled_loss:
                    scaled_loss.backward()
                torch.nn.utils.clip_grad_norm_(amp.master_params(optimizer), args.max_grad_norm)
            else:
                loss.backward()
                torch.nn.utils.clip_grad_norm_(model.parameters(), args.max_grad_norm)

            tr_loss += loss.item()
            if (step + 1) % args.gradient_accumulation_steps == 0:
                scheduler.step()  # Update learning rate schedule
                optimizer.step()
                model.zero_grad()
                global_step += 1

                if args.local_rank in [-1, 0] and args.logging_steps > 0 and global_step % args.logging_steps == 0:
                    results = {}

                    # Log metrics
                    logits_ix = 1 if args.model_type == "bert" else 7
                    logits = outputs[logits_ix]
                    preds = np.argmax(logits.detach().cpu().numpy(), axis=1)
                    out_label_ids = inputs['labels'].detach().cpu().numpy()

                    result = compute_metrics(args.task_name, preds, out_label_ids)
                    results.update(result)

                    for key, value in results.items():
                        wandb.log({'train_{}'.format(key): value})

                    wandb.log({"train_loss": (tr_loss - logging_loss) / args.logging_steps})
                    wandb.log({"train_lr": scheduler.get_lr()[0]})
                    logging_loss = tr_loss

            if args.max_steps > 0 and global_step > args.max_steps:
                epoch_iterator.close()
                break

        if args.evaluate_during_training:
            # Only evaluate when single GPU otherwise metrics may not average well
            results = evaluate(args, model, tokenizer)
            for key, value in results.items():
                wandb.log({'eval_{}'.format(key): value})

        if args.local_rank in [-1, 0]:
            # Save model checkpoint
            output_dir = os.path.join(args.output_dir, 'checkpoint-{}'.format(epoch_ix))
            if not os.path.exists(output_dir):
                os.makedirs(output_dir)
            model_to_save = model.module if hasattr(model, 'module') else model  # Take care of distributed/parallel training
            model_to_save.save_pretrained(output_dir)
            torch.save(args, os.path.join(output_dir, 'training_args.bin'))
            logger.info("Saving model checkpoint to %s", output_dir)

        if args.max_steps > 0 and global_step > args.max_steps:
            train_iterator.close()
            break

    return global_step, tr_loss / global_step


def evaluate(args, model, tokenizer, prefix=""):
    # Loop to handle MNLI double evaluation (matched, mis-matched)
    eval_task_names = (args.task_name,)
    eval_outputs_dirs = (args.output_dir,)

    results = {}
    cnt = 0
    for eval_task, eval_output_dir in zip(eval_task_names, eval_outputs_dirs):
        eval_dataset = load_and_cache_examples(args, eval_task, tokenizer, evaluate=True)

        if not os.path.exists(eval_output_dir) and args.local_rank in [-1, 0]:
            os.makedirs(eval_output_dir)

        args.eval_batch_size = args.per_gpu_eval_batch_size * max(1, args.n_gpu)
        # Note that DistributedSampler samples randomly
        eval_sampler = SequentialSampler(eval_dataset) if args.local_rank == -1 else DistributedSampler(eval_dataset)
        eval_dataloader = DataLoader(eval_dataset, sampler=eval_sampler, batch_size=args.eval_batch_size)

        # Eval!
        logger.info("***** Running evaluation {} *****".format(prefix))
        logger.info("  Num examples = %d", len(eval_dataset))
        logger.info("  Batch size = %d", args.eval_batch_size)

        eval_loss = 0.0
        nb_eval_steps = 0
        preds = None
        out_label_ids = None

        for batch in tqdm(eval_dataloader, desc="Evaluating"):
            model.eval()
            batch = tuple(t.to(args.device) for t in batch)

            with torch.no_grad():
                inputs = make_model_input(args, batch)
                outputs = model(**inputs)

                # monitoring
                tmp_eval_loss = outputs[0]
                logits_ix = 1 if args.model_type == "bert" else 7
                logits = outputs[logits_ix]
                eval_loss += tmp_eval_loss.mean().item()
                nb_eval_steps += 1

            if preds is None:
                preds = logits.detach().cpu().numpy()
                out_label_ids = inputs['labels'].detach().cpu().numpy()
            else:
                preds = np.append(preds, logits.detach().cpu().numpy(), axis=0)
                out_label_ids = np.append(out_label_ids, inputs['labels'].detach().cpu().numpy(), axis=0)

        preds = np.argmax(preds, axis=1)
        result = compute_metrics(args.task_name, preds, out_label_ids)
        eval_loss = eval_loss / nb_eval_steps
        result["loss"] = eval_loss
        results.update(result)

        output_eval_file = os.path.join(eval_output_dir, "eval_results.txt")
        with open(output_eval_file, "w") as writer:
            logger.info("***** Eval results {} *****".format(prefix))
            for key in sorted(results.keys()):
                logger.info("  %s = %s", key, str(result[key]))
                writer.write("%s = %s\n" % (key, str(result[key])))

    return results


def test(args, model, tokenizer, prefix=""):
    """
    An edited version of `evaluate` that saves the predictions and does not write
    the results file.
    """
    # Loop to handle MNLI double evaluation (matched, mis-matched)
    eval_task_names = (args.task_name,)
    eval_outputs_dirs = (args.output_dir,)

    results = {}
    cnt = 0
    for eval_task, eval_output_dir in zip(eval_task_names, eval_outputs_dirs):
        eval_dataset = load_and_cache_examples(args, eval_task, tokenizer, evaluate=True)

        if not os.path.exists(eval_output_dir) and args.local_rank in [-1, 0]:
            os.makedirs(eval_output_dir)

        args.eval_batch_size = args.per_gpu_eval_batch_size * max(1, args.n_gpu)
        # Note that DistributedSampler samples randomly
        eval_sampler = SequentialSampler(eval_dataset) if args.local_rank == -1 else DistributedSampler(eval_dataset)
        eval_dataloader = DataLoader(eval_dataset, sampler=eval_sampler, batch_size=args.eval_batch_size)

        # Eval!
        logger.info("***** Running evaluation {} *****".format(prefix))
        logger.info("  Num examples = %d", len(eval_dataset))
        logger.info("  Batch size = %d", args.eval_batch_size)

        eval_loss = 0.0
        nb_eval_steps = 0
        preds = None
        out_label_ids = None
        probs = None

        for batch in tqdm(eval_dataloader, desc="Evaluating"):
            model.eval()
            batch = tuple(t.to(args.device) for t in batch)

            with torch.no_grad():
                inputs = make_model_input(args, batch)
                outputs = model(**inputs)

                # monitoring
                tmp_eval_loss = outputs[0]
                logits_ix = 1 if args.model_type == "bert" else 7
                logits = outputs[logits_ix]
                eval_loss += tmp_eval_loss.mean().item()
                nb_eval_steps += 1

            if preds is None:
                preds = logits.detach().cpu().numpy()
                probs = torch.softmax(logits, dim=1).detach().cpu().numpy()
                out_label_ids = inputs['labels'].detach().cpu().numpy()
            else:
                preds = np.append(preds, logits.detach().cpu().numpy(), axis=0)
                probs = np.append(probs, torch.softmax(logits, dim=1).detach().cpu().numpy(), axis=0)
                out_label_ids = np.append(out_label_ids, inputs['labels'].detach().cpu().numpy(), axis=0)

        pred_labels = np.argmax(preds, axis=1)
        result = compute_metrics(args.task_name, pred_labels, out_label_ids)
        eval_loss = eval_loss / nb_eval_steps
        result["loss"] = eval_loss
        result["labels"] = pred_labels
        result["scores"] = probs[:, 1]
        results.update(result)

    return results


def load_and_cache_examples(args, task, tokenizer, evaluate=False):
    if args.local_rank not in [-1, 0]:
        torch.distributed.barrier()  # Make sure only the first process in distributed training process the dataset, and the others will use the cache

    processor = processors[task]()
    output_mode = output_modes[task]
    # Load data features from cache or dataset file
    cached_features_file = os.path.join(args.data_dir, 'cached_{}_{}_{}_{}'.format(
        'dev' if evaluate else 'train',
        list(filter(None, args.model_name_or_path.split('/'))).pop(),
        str(args.max_seq_length),
        str(task)))
    if os.path.exists(cached_features_file):
        logger.info("Loading features from cached file %s", cached_features_file)
        features = torch.load(cached_features_file)
    else:
        logger.info("Creating features from dataset file at %s", args.data_dir)
        label_list = processor.get_labels()
        examples = processor.get_dev_examples(args.data_dir) if evaluate else processor.get_train_examples(args.data_dir)
        features = convert_examples_to_features(examples, label_list, args.max_seq_length, tokenizer, output_mode,
            cls_token_at_end=bool(args.model_type in ['xlnet']),            # xlnet has a cls token at the end
            cls_token=tokenizer.cls_token,
            cls_token_segment_id=2 if args.model_type in ['xlnet'] else 0,
            sep_token=tokenizer.sep_token,
            sep_token_extra=bool(args.model_type in ['roberta']),
            pad_on_left=bool(args.model_type in ['xlnet']),                 # pad on the left for xlnet
            pad_token=tokenizer.convert_tokens_to_ids([tokenizer.pad_token])[0],
            pad_token_segment_id=4 if args.model_type in ['xlnet'] else 0)
        if args.local_rank in [-1, 0]:
            logger.info("Saving features into cached file %s", cached_features_file)
            torch.save(features, cached_features_file)

    if args.local_rank == 0:
        torch.distributed.barrier()  # Make sure only the first process in distributed training process the dataset, and the others will use the cache

    # Convert to Tensors and build dataset
    all_input_ids = torch.tensor([f.input_ids for f in features], dtype=torch.long)
    all_input_mask = torch.tensor([f.input_mask for f in features], dtype=torch.long)
    all_segment_ids = torch.tensor([f.segment_ids for f in features], dtype=torch.long)
    all_ext_mask = torch.tensor([f.extraction_mask for f in features], dtype=torch.float)
    all_ext_start_ids = torch.tensor([f.extraction_start_ids for f in features], dtype=torch.long)
    all_ext_end_ids = torch.tensor([f.extraction_end_ids for f in features], dtype=torch.long)
    all_aug_mask = torch.tensor([f.augmentation_mask for f in features], dtype=torch.float)
    all_aug_start_ids = torch.tensor([f.augmentation_start_ids for f in features], dtype=torch.long)
    all_aug_end_ids = torch.tensor([f.augmentation_end_ids for f in features], dtype=torch.long)

    if output_mode == "classification":
        all_label_ids = torch.tensor([f.label_id for f in features], dtype=torch.long)
    elif output_mode == "regression":
        all_label_ids = torch.tensor([f.label_id for f in features], dtype=torch.float)

    dataset = TensorDataset(all_input_ids, all_input_mask, all_segment_ids, all_label_ids,
                            all_ext_mask, all_ext_start_ids, all_ext_end_ids,
                            all_aug_mask, all_aug_start_ids, all_aug_end_ids)
    return dataset


def main():
    parser = argparse.ArgumentParser()

    ## Required parameters
    parser.add_argument("--data_dir", default=None, type=str, required=True,
                        help="The input data dir. Should contain the .tsv files (or other data files) for the task.")
    parser.add_argument("--model_type", default=None, type=str, required=True,
                        help="Model type selected in the list: " + ", ".join(MODEL_CLASSES.keys()))
    parser.add_argument("--model_name_or_path", default=None, type=str, required=True,
                        help="Path to pre-trained model or shortcut name selected in the list: " + ", ".join(ALL_MODELS))
    parser.add_argument("--task_name", default=None, type=str, required=True,
                        help="The name of the task to train selected in the list: " + ", ".join(processors.keys()))
    parser.add_argument("--output_dir", default=None, type=str, required=True,
                        help="The output directory where the model predictions and checkpoints will be written.")
    parser.add_argument("--train_from_scratch", action='store_true',
			help="Whether to run training without loading pretrained weights.")

    ## Other parameters
    parser.add_argument("--config_name", default="", type=str,
                        help="Pretrained config name or path if not the same as model_name")
    parser.add_argument("--tokenizer_name", default="", type=str,
                        help="Pretrained tokenizer name or path if not the same as model_name")
    parser.add_argument("--cache_dir", default="", type=str,
                        help="Where do you want to store the pre-trained models downloaded from s3")
    parser.add_argument("--max_seq_length", default=512, type=int,
                        help="The maximum total input sequence length after tokenization. Sequences longer "
                             "than this will be truncated, sequences shorter will be padded.")
    parser.add_argument("--do_train", action='store_true',
                        help="Whether to run training.")
    parser.add_argument("--do_eval", action='store_true',
                        help="Whether to run eval on the dev set.")
    parser.add_argument("--evaluate_during_training", action='store_true',
                        help="Run evaluation during training at each logging step.")
    parser.add_argument("--do_lower_case", action='store_true',
                        help="Set this flag if you are using an uncased model.")

    parser.add_argument("--per_gpu_train_batch_size", default=8, type=int,
                        help="Batch size per GPU/CPU for training.")
    parser.add_argument("--per_gpu_eval_batch_size", default=8, type=int,
                        help="Batch size per GPU/CPU for evaluation.")
    parser.add_argument('--gradient_accumulation_steps', type=int, default=1,
                        help="Number of updates steps to accumulate before performing a backward/update pass.")
    parser.add_argument("--learning_rate", default=5e-5, type=float,
                        help="The initial learning rate for Adam.")
    parser.add_argument("--loss_lambda", default=0.1, type=float,
                        help="The lambda parameter for loss mixing.")
    parser.add_argument("--weight_decay", default=0.0, type=float,
                        help="Weight deay if we apply some.")
    parser.add_argument("--adam_epsilon", default=1e-8, type=float,
                        help="Epsilon for Adam optimizer.")
    parser.add_argument("--max_grad_norm", default=1.0, type=float,
                        help="Max gradient norm.")
    parser.add_argument("--num_train_epochs", default=3.0, type=float,
                        help="Total number of training epochs to perform.")
    parser.add_argument("--max_steps", default=-1, type=int,
                        help="If > 0: set total number of training steps to perform. Override num_train_epochs.")
    parser.add_argument("--warmup_steps", default=0, type=int,
                        help="Linear warmup over warmup_steps.")

    parser.add_argument('--logging_steps', type=int, default=100,
                        help="Log every X updates steps.")
    parser.add_argument('--save_steps', type=int, default=50,
                        help="Save checkpoint every X updates steps.")
    parser.add_argument("--eval_all_checkpoints", action='store_true',
                        help="Evaluate all checkpoints starting with the same prefix as model_name ending and ending with step number")
    parser.add_argument("--no_cuda", action='store_true',
                        help="Avoid using CUDA when available")
    parser.add_argument('--overwrite_output_dir', action='store_true',
                        help="Overwrite the content of the output directory")
    parser.add_argument('--overwrite_cache', action='store_true',
                        help="Overwrite the cached training and evaluation sets")
    parser.add_argument('--seed', type=int, default=42,
                        help="random seed for initialization")

    parser.add_argument('--fp16', action='store_true',
                        help="Whether to use 16-bit (mixed) precision (through NVIDIA apex) instead of 32-bit")
    parser.add_argument('--fp16_opt_level', type=str, default='O1',
                        help="For fp16: Apex AMP optimization level selected in ['O0', 'O1', 'O2', and 'O3']."
                             "See details at https://nvidia.github.io/apex/amp.html")
    parser.add_argument("--local_rank", type=int, default=-1,
                        help="For distributed training: local_rank")

    parser.add_argument("--do_test", action='store_true',
                        help="Whether to run eval on the test set.")
    parser.add_argument("--output-file", required=True)

    args = parser.parse_args()

    if os.path.exists(args.output_dir) and os.listdir(args.output_dir) and args.do_train and not args.overwrite_output_dir:
        raise ValueError("Output directory ({}) already exists and is not empty. Use --overwrite_output_dir to overcome.".format(args.output_dir))

    # Setup CUDA, GPU & distributed training
    if args.local_rank == -1 or args.no_cuda:
        device = torch.device("cuda" if torch.cuda.is_available() and not args.no_cuda else "cpu")
        args.n_gpu = torch.cuda.device_count()
    else:  # Initializes the distributed backend which will take care of sychronizing nodes/GPUs
        torch.cuda.set_device(args.local_rank)
        device = torch.device("cuda", args.local_rank)
        torch.distributed.init_process_group(backend='nccl')
        args.n_gpu = 1
    args.device = device

    # Setup logging
    logging.basicConfig(format = '%(asctime)s - %(levelname)s - %(name)s -   %(message)s',
                        datefmt = '%m/%d/%Y %H:%M:%S',
                        level = logging.INFO if args.local_rank in [-1, 0] else logging.WARN)
    logger.warning("Process rank: %s, device: %s, n_gpu: %s, distributed training: %s, 16-bits training: %s",
                    args.local_rank, device, args.n_gpu, bool(args.local_rank != -1), args.fp16)

    # Set seed
    set_seed(args)

    # Prepare GLUE task
    args.task_name = args.task_name.lower()
    if args.task_name not in processors:
        raise ValueError("Task not found: %s" % (args.task_name))
    processor = processors[args.task_name]()
    args.output_mode = output_modes[args.task_name]
    label_list = processor.get_labels()
    num_labels = len(label_list)

    # Load pretrained model and tokenizer
    if args.local_rank not in [-1, 0]:
        torch.distributed.barrier()  # Make sure only the first process in distributed training will download model & vocab

    args.model_type = args.model_type.lower()
    config_class, model_class, tokenizer_class = MODEL_CLASSES[args.model_type]
    config = config_class.from_pretrained(args.config_name if args.config_name else args.model_name_or_path, num_labels=num_labels, finetuning_task=args.task_name)
    tokenizer = tokenizer_class.from_pretrained(args.tokenizer_name if args.tokenizer_name else args.model_name_or_path, do_lower_case=args.do_lower_case)
    if args.train_from_scratch:
        logger.info("Training model from scratch.")
        model = model_class(config=config)
    else:
        logger.info("Loading model from checkpoint.")
        model = model_class.from_pretrained(args.model_name_or_path, from_tf=bool('.ckpt' in args.model_name_or_path), config=config)

    if args.local_rank == 0:
        torch.distributed.barrier()  # Make sure only the first process in distributed training will download model & vocab

    model.to(args.device)

    wandb.watch(model)
    logger.info("Training/evaluation parameters %s", args)


    # Training
    if args.do_train:
        train_dataset = load_and_cache_examples(args, args.task_name, tokenizer, evaluate=False)
        global_step, tr_loss = train(args, train_dataset, model, tokenizer)
        logger.info(" global_step = %s, average loss = %s", global_step, tr_loss)


    # Saving best-practices: if you use defaults names for the model, you can reload it using from_pretrained()
    if args.do_train and (args.local_rank == -1 or torch.distributed.get_rank() == 0):
        # Create output directory if needed
        if not os.path.exists(args.output_dir) and args.local_rank in [-1, 0]:
            os.makedirs(args.output_dir)

        logger.info("Saving model checkpoint to %s", args.output_dir)
        # Save a trained model, configuration and tokenizer using `save_pretrained()`.
        # They can then be reloaded using `from_pretrained()`
        model_to_save = model.module if hasattr(model, 'module') else model  # Take care of distributed/parallel training
        model_to_save.save_pretrained(args.output_dir)
        tokenizer.save_pretrained(args.output_dir)

        # Good practice: save your training arguments together with the trained model
        torch.save(args, os.path.join(args.output_dir, 'training_args.bin'))

        # Load a trained model and vocabulary that you have fine-tuned
        model = model_class.from_pretrained(args.output_dir)
        tokenizer = tokenizer_class.from_pretrained(args.output_dir)
        model.to(args.device)


    # Evaluation
    results = {}
    if args.do_eval and args.local_rank in [-1, 0]:
        checkpoints = [args.output_dir]
        if args.eval_all_checkpoints:
            checkpoints = list(os.path.dirname(c) for c in sorted(glob.glob(args.output_dir + '/**/' + WEIGHTS_NAME, recursive=True)))
            logging.getLogger("pytorch_transformers.modeling_utils").setLevel(logging.WARN)  # Reduce logging
        logger.info("Evaluate the following checkpoints: %s", checkpoints)
        for checkpoint in checkpoints:
            global_step = checkpoint.split('-')[-1] if len(checkpoints) > 1 else ""
            model = model_class.from_pretrained(checkpoint)
            model.to(args.device)
            result = evaluate(args, model, tokenizer, prefix=global_step)
            result = dict((k + '_{}'.format(global_step), v) for k, v in result.items())
            results.update(result)

    results = {}
    if args.do_test:
        checkpoints = [args.output_dir]
        if args.eval_all_checkpoints:
            checkpoints = list(os.path.dirname(c) for c in sorted(glob.glob(args.output_dir + '/**/' + WEIGHTS_NAME, recursive=True)))
            assert len(checkpoints) == 1
            logging.getLogger("pytorch_transformers.modeling_utils").setLevel(logging.WARN)  # Reduce logging
        logger.info("Evaluate the following checkpoints: %s", checkpoints)
        for checkpoint in checkpoints:
            global_step = checkpoint.split('-')[-1] if len(checkpoints) > 1 else ""
            model = model_class.from_pretrained(checkpoint)
            model.to(args.device)
            result = test(args, model, tokenizer, prefix=global_step)
            result = dict((k + '_{}'.format(global_step), v) for k, v in result.items())
            results.update(result)

        with open(args.output_file, "w") as out:
            for label, score in zip(results["labels_"], results["scores_"]):
                out.write(json.dumps({
                    "score": float(score),
                    "label": int(label),
                }) + "\n")


    return results


if __name__ == "__main__":
    main()



import os

VERSION = "1.0"
MODEL_NAME = os.path.basename(os.path.dirname(__file__))
DOCKERHUB_REPRO = f"danieldeutsch/{MODEL_NAME}"
DEFAULT_IMAGE = f"{DOCKERHUB_REPRO}:{VERSION}"
AUTOMATICALLY_PUBLISH = True

from repro.models.deutsch2021.dataset_reader import (
    Deutsch2021QuestionAnsweringEvaluationDatasetReader,
)
from repro.models.deutsch2021.models import (
    QAEval,
    QAEvalQuestionAnsweringModel,
    QAEvalQuestionGenerationModel,
)
from repro.models.deutsch2021.setup import Deutsch2021SetupSubcommand



import json
import logging
from typing import Any, Dict, List, Tuple, Union

from repro.common import util
from repro.common.docker import DockerContainer
from repro.common.io import read_jsonl_file
from repro.data.types import MetricsType, TextType
from repro.models import Model
from repro.models.goyal2020 import DEFAULT_IMAGE, MODEL_NAME

QAPairsType = List[List[Dict[str, Any]]]

logger = logging.getLogger(__name__)


@Model.register(f"{MODEL_NAME}-dae")
class DAE(Model):
    def __init__(
        self,
        image: str = DEFAULT_IMAGE,
        model: str = "dae_w_syn",
        device: int = 0,
        sleep: int = 1,
    ):
        """
        Parameters
        ----------
        image : str, default="goyal2020"
            The name of the Docker image
        model : str, default="dae_w_syn"
            The name of the pre-trained DAE model to use
        device : int, default=0
            The ID of the GPU to use, -1 if CPU
        sleep : int, default=1
            The number of seconds to sleep while the Stanford CoreNLP server initializes
        """
        self.image = image
        self.model = model
        self.device = device
        self.sleep = sleep

    def predict(
        self, candidate: TextType, sources: List[TextType], **kwargs
    ) -> MetricsType:
        return self.predict_batch(
            [{"candidate": candidate, "sources": sources}], **kwargs
        )[0]

    def predict_batch(
        self,
        inputs: List[Dict[str, Union[TextType, List[TextType]]]],
        **kwargs,
    ) -> Tuple[MetricsType, List[MetricsType]]:
        logger.info(f"Calculating DAE for {len(inputs)} inputs with model {self.model}")

        candidates = [inp["candidate"] for inp in inputs]
        sources_list = [inp["sources"] for inp in inputs]

        # DAE only accepts single source documents
        sources = util.check_for_single_texts(sources_list)

        # Make sure all are type `str`
        candidates = [util.flatten(candidate) for candidate in candidates]
        sources = [util.flatten(source) for source in sources]

        with DockerContainer(self.image) as backend:
            host_input_file = f"{backend.host_dir}/input.jsonl"
            container_input_file = f"{backend.container_dir}/input.jsonl"
            with open(host_input_file, "w") as out:
                for candidate, source in zip(candidates, sources):
                    out.write(
                        json.dumps(
                            {
                                "candidate": candidate,
                                "source": source,
                            }
                        )
                        + "\n"
                    )

            host_output_file = f"{backend.host_dir}/output.jsonl"
            container_output_file = f"{backend.container_dir}/output.jsonl"

            commands = []
            cuda = self.device != -1
            if cuda:
                score_device = 0
                commands.append(f"export CUDA_VISIBLE_DEVICES={self.device}")
            else:
                score_device = -1

            commands.append(
                f"sh run.sh"
                f"  {container_input_file}"
                f"  {container_output_file}"
                f"  {self.model}"
                f"  {score_device}"
                f"  {self.sleep}"
            )

            command = " && ".join(commands)
            backend.run_command(
                command=command,
                cuda=cuda,
                network_disabled=False,
            )

            micro_metrics = read_jsonl_file(host_output_file)
            macro_metrics = util.average_dicts(micro_metrics)
            return macro_metrics, micro_metrics



import argparse

from overrides import overrides

from repro import MODELS_ROOT
from repro.commands.subcommand import SetupSubcommand
from repro.common.docker import BuildDockerImageSubcommand, build_image
from repro.models.zhang2020 import DEFAULT_IMAGE, MODEL_NAME


@SetupSubcommand.register(MODEL_NAME)
class Zhang2020SetupSubcommand(BuildDockerImageSubcommand):
    def __init__(self) -> None:
        super().__init__(f"{MODELS_ROOT}/{MODEL_NAME}", DEFAULT_IMAGE)

    @overrides
    def add_subparser(self, model: str, parser: argparse._SubParsersAction):
        description = f'Build a Docker image for model "{model}"'
        self.parser = parser.add_parser(
            model, description=description, help=description
        )
        self.parser.add_argument(
            "--image-name",
            default=DEFAULT_IMAGE,
            help="The name of the image to build",
        )
        self.parser.add_argument(
            "--models",
            nargs="+",
            help="Indicates which models should be downloaded",
            default=["roberta-large"],
        )
        self.parser.add_argument(
            "--silent",
            action="store_true",
            help="Silences the output from the build command",
        )
        self.parser.set_defaults(subfunc=self.run)

    @overrides
    def run(self, args):
        build_args = {
            "MODELS": " ".join(args.models),
        }
        build_image(
            self.root, args.image_name, build_args=build_args, silent=args.silent
        )



import json
import logging
from typing import Dict, List, Tuple, Union

from repro.common import util
from repro.common.docker import DockerContainer
from repro.common.io import read_jsonl_file
from repro.data.types import MetricsType, TextType
from repro.models import Model
from repro.models.kryscinski2019 import DEFAULT_IMAGE, MODEL_NAME

logger = logging.getLogger(__name__)


class _Kryscinski2019Model(Model):
    def __init__(
        self,
        is_factccx: bool,
        image: str = DEFAULT_IMAGE,
        device: int = 0,
        batch_size: int = 8,
    ):
        self.is_factccx = is_factccx
        if is_factccx:
            self.name = "FactCCX"
            self.model = "factccx-checkpoint"
            self.model_type = "pbert"
        else:
            self.name = "FactCC"
            self.model = "factcc-checkpoint"
            self.model_type = "bert"
        self.image = image
        self.device = device
        self.batch_size = batch_size

    def predict(
        self, candidate: TextType, sources: List[TextType], **kwargs
    ) -> MetricsType:
        return self.predict_batch(
            [{"candidate": candidate, "sources": sources}], **kwargs
        )[0]

    def predict_batch(
        self,
        inputs: List[Dict[str, Union[TextType, List[TextType]]]],
        **kwargs,
    ) -> Tuple[MetricsType, List[MetricsType]]:
        logger.info(f"Calculating {self.name} for {len(inputs)} inputs with model")

        candidates = [inp["candidate"] for inp in inputs]
        sources_list = [inp["sources"] for inp in inputs]

        # FactCC only accepts single source documents
        sources = util.check_for_single_texts(sources_list)

        # Make sure all are type `str`
        candidates = [util.flatten(candidate) for candidate in candidates]
        sources = [util.flatten(source) for source in sources]

        with DockerContainer(self.image) as backend:
            host_input_file = f"{backend.host_dir}/data-dev.jsonl"
            with open(host_input_file, "w") as out:
                for i, (candidate, source) in enumerate(zip(candidates, sources)):
                    out.write(
                        json.dumps(
                            {
                                "id": str(i),
                                "claim": candidate,
                                "text": source,
                                "label": "CORRECT",  # dummy label
                            }
                        )
                        + "\n"
                    )

            host_output_file = f"{backend.host_dir}/output.jsonl"
            container_output_file = f"{backend.container_dir}/output.jsonl"

            commands = []
            cuda = self.device != -1
            if cuda:
                commands.append(f"export CUDA_VISIBLE_DEVICES={self.device}")

            commands.append("cd factCC/modeling")

            # The run.py code uses wandb for logging, but the code will fail
            # if we run it without first disabling wandb
            commands.append("wandb disabled")

            score_command = (
                f"python run_test.py"
                f"  --task_name factcc_annotated"
                f"  --do_test"
                f"  --eval_all_checkpoints"
                f"  --do_lower_case"
                f"  --overwrite_cache"
                f"  --max_seq_length 512"
                f"  --per_gpu_eval_batch_size {self.batch_size}"
                f"  --model_type {self.model_type}"
                f"  --model_name_or_path bert-base-uncased"
                f"  --data_dir {backend.container_dir}"
                f"  --output_dir ../../{self.model}"
                f"  --output-file {container_output_file}"
            )
            if not cuda:
                score_command += " --no_cuda"
            commands.append(score_command)

            command = " && ".join(commands)
            backend.run_command(
                command=command,
                cuda=cuda,
                network_disabled=True,
            )

            micro_metrics = read_jsonl_file(host_output_file)
            micro_metrics = [{self.name.lower(): metrics} for metrics in micro_metrics]
            macro_metrics = util.average_dicts(micro_metrics)
            return macro_metrics, micro_metrics


@Model.register(f"{MODEL_NAME}-factcc")
class FactCC(_Kryscinski2019Model):
    def __init__(self, **kwargs) -> None:
        super().__init__(is_factccx=False, **kwargs)


@Model.register(f"{MODEL_NAME}-factccx")
class FactCCX(_Kryscinski2019Model):
    def __init__(self, **kwargs) -> None:
        super().__init__(is_factccx=True, **kwargs)



import argparse
import importlib
import pkgutil
from overrides import overrides

from repro.commands.subcommand import RootSubcommand
from repro.common.docker import pull_image
from repro.common.logging import prepare_global_logging


@RootSubcommand.register("pull")
class PullImageSubcommand(RootSubcommand):
    @overrides
    def add_subparser(self, parser: argparse._SubParsersAction):
        description = "Pull a model's default Docker image from Docker Hub"
        self.parser = parser.add_parser(
            "pull", description=description, help=description
        )

        # Find all of the modules that have default Docker Hub repos
        self.images = {}
        for module_info in pkgutil.iter_modules(["repro/models"]):
            module_name = f"repro.models.{module_info.name}"
            module = importlib.import_module(module_name)

            try:
                model = module.MODEL_NAME
                image = module.DEFAULT_IMAGE
                self.images[model] = image
            except AttributeError:
                # We can't find a default image
                pass

        choices = sorted(self.images.keys())
        self.parser.add_argument(
            "model",
            help="The name of the model to pull the Docker image for",
            choices=choices,
        )

        self.parser.set_defaults(func=self.run)

    @overrides
    def run(self, args):
        prepare_global_logging()
        image = self.images[args.model]
        pull_image(image)



import json
import logging
import os
from typing import Dict, List, Tuple, Union

from repro.common import TemporaryDirectory, util
from repro.common.docker import make_volume_map, run_command
from repro.common.io import read_jsonl_file
from repro.data.types import MetricsType, TextType
from repro.models import Model
from repro.models.zhang2020 import DEFAULT_IMAGE, MODEL_NAME

logger = logging.getLogger(__name__)


@Model.register(f"{MODEL_NAME}-bertscore")
class BERTScore(Model):
    def __init__(
        self,
        image: str = DEFAULT_IMAGE,
        model: str = None,
        device: int = 0,
        batch_size: int = 64,
        language: str = "en",
    ):
        self.image = image
        self.model = model
        self.device = device
        self.batch_size = batch_size
        self.language = language

    def predict(
        self,
        candidate: TextType,
        references: List[TextType],
        **kwargs,
    ) -> MetricsType:
        return self.predict_batch(
            [{"candidate": candidate, "references": references}], **kwargs
        )[0]

    def predict_batch(
        self, inputs: List[Dict[str, Union[TextType, List[TextType]]]], **kwargs
    ) -> Tuple[MetricsType, List[MetricsType]]:
        logger.info(
            f"Calculating BERTScore with image {self.image} on {len(inputs)} inputs."
        )

        candidates = [inp["candidate"] for inp in inputs]
        references_list = [inp["references"] for inp in inputs]

        # The each candidate and reference must be `str`, not `List[str]`
        candidates = [util.flatten(candidate) for candidate in candidates]
        references_list = [
            [util.flatten(reference) for reference in references]
            for references in references_list
        ]

        # The metric code will run if we provide it an empty input, but we want to
        # assign a value of 0 to that candidate. Remove the empty inputs here, then
        # add 0 values at the end
        empty_indices, candidates, references_list = util.remove_empty_inputs(
            candidates, references_list
        )

        with TemporaryDirectory() as temp:
            host_input_dir = f"{temp}/input"
            host_output_dir = f"{temp}/output"
            volume_map = make_volume_map(host_input_dir, host_output_dir)
            container_input_dir = volume_map[host_input_dir]
            container_output_dir = volume_map[host_output_dir]

            host_input_file = f"{host_input_dir}/input.jsonl"
            container_input_file = f"{container_input_dir}/input.jsonl"
            os.makedirs(host_input_dir)
            with open(host_input_file, "w") as out:
                for candidate, references in zip(candidates, references_list):
                    out.write(
                        json.dumps(
                            {
                                "candidate": candidate,
                                "references": references,
                            }
                        )
                        + "\n"
                    )

            host_output_file = f"{host_output_dir}/output.jsonl"
            container_output_file = f"{container_output_dir}/output.jsonl"

            cuda = self.device != -1
            commands = []
            if cuda:
                commands.append(f"export CUDA_VISIBLE_DEVICES={self.device}")
                predict_device = 0
            else:
                predict_device = -1

            score_command = (
                f"python score.py"
                f"  --input-file {container_input_file}"
                f"  --cuda-device {predict_device}"
                f"  --batch-size {self.batch_size}"
                f"  --output-file {container_output_file}"
            )
            if self.model is not None:
                score_command += f" --model-name {self.model}"
            if self.language is not None:
                score_command += f"  --language {self.language}"
            commands.append(score_command)

            command = " && ".join(commands)
            os.makedirs(host_output_dir)
            run_command(
                self.image,
                command,
                volume_map=volume_map,
                cuda=cuda,
            )

            micro_metrics = read_jsonl_file(host_output_file)
            micro_metrics = [{"bertscore": scores} for scores in micro_metrics]

            # Insert default metric values for inputs which were empty. All of the
            # scored inputs should have the same keys, so we can get a default set
            # of metrics from just the first set of metrics
            empty_value = util.get_default_dict(micro_metrics[0], default=0.0)
            micro_metrics = util.insert_empty_values(
                micro_metrics, empty_indices, empty_value
            )

            macro_metrics = util.average_dicts(micro_metrics)
            return macro_metrics, micro_metrics



from repro import MODELS_ROOT
from repro.commands.subcommand import SetupSubcommand
from repro.common.docker import BuildDockerImageSubcommand
from repro.models.deutsch2021 import DEFAULT_IMAGE, MODEL_NAME


@SetupSubcommand.register(MODEL_NAME)
class Deutsch2021SetupSubcommand(BuildDockerImageSubcommand):
    def __init__(self) -> None:
        super().__init__(f"{MODELS_ROOT}/{MODEL_NAME}", DEFAULT_IMAGE)



import json
import unittest
from parameterized import parameterized

from repro.models.zhang2020 import BERTScore
from repro.testing import FIXTURES_ROOT as REPRO_FIXTURES_ROOT
from repro.testing import assert_dicts_approx_equal, get_testing_device_parameters

from . import FIXTURES_ROOT


class TestZhang2020Models(unittest.TestCase):
    def setUp(self) -> None:
        self.examples = json.load(
            open(f"{REPRO_FIXTURES_ROOT}/multiling2011/data.json", "r")
        )
        self.expected = json.load(open(f"{FIXTURES_ROOT}/expected.json", "r"))
        self.bertscore_examples = json.load(
            open(f"{FIXTURES_ROOT}/bertscore-unittests.json", "r")
        )

    @parameterized.expand(get_testing_device_parameters())
    def test_bertscore(self, device: int):
        model = BERTScore(device=device)
        inputs = [
            {"candidate": inp["candidate"], "references": inp["references"]}
            for inp in self.examples
        ]
        expected_macro = self.expected["macro"]
        expected_micro = self.expected["micro"]
        actual_macro, actual_micro = model.predict_batch(inputs)

        assert_dicts_approx_equal(expected_macro, actual_macro, abs=1e-4)
        assert len(expected_micro) == len(actual_micro)
        for expected, actual in zip(expected_micro, actual_micro):
            assert_dicts_approx_equal(expected, actual, abs=1e-4)

    @parameterized.expand(get_testing_device_parameters())
    def test_bertscore_unittests(self, device: int):
        # This tests the examples in the bert_score repo unit tests
        model = BERTScore(device=device)
        inputs = self.bertscore_examples["inputs"]
        expected_micro = self.bertscore_examples["micro"]
        _, actual_micro = model.predict_batch(inputs)

        assert len(expected_micro) == len(actual_micro)
        for expected, actual in zip(expected_micro, actual_micro):
            assert_dicts_approx_equal(expected, actual, abs=1e-4)

    @parameterized.expand(get_testing_device_parameters())
    def test_empty_input(self, device: int):
        # Tests to make sure there's a dict with 0s returned. Without
        # directly handling the empty inputs, this test does not pass. Also
        # ensures the non-empty inputs values didn't change
        model = BERTScore(device=device)

        input1 = {"candidate": "The first", "references": ["The first reference"]}
        input2 = {"candidate": "", "references": ["The first reference"]}
        input3 = {"candidate": "The third", "references": ["The first reference"]}

        inputs_with_empty = [input1, input2, input3]
        inputs_without_empty = [input1, input3]

        _, micro_with_empty = model.predict_batch(inputs_with_empty)
        _, micro_without_empty = model.predict_batch(inputs_without_empty)

        # Make sure non-empty didn't change
        assert_dicts_approx_equal(micro_with_empty[0], micro_without_empty[0])
        assert_dicts_approx_equal(micro_with_empty[2], micro_without_empty[1])

        # Make sure empty has 0s
        for value in micro_with_empty[1]["bertscore"].values():
            assert value == 0.0



import argparse
import json
import tarfile
from collections import defaultdict
from sklearn.metrics import balanced_accuracy_score, f1_score
from tqdm import tqdm
from typing import List, Tuple

from repro.common.io import read_jsonl_file
from repro.models.kryscinski2019 import FactCC, FactCCX


def parse_story_file(content):
    """
    Remove article highlights and unnecessary white characters.
    """
    content_raw = content.split("@highlight")[0]
    content = " ".join(filter(None, [x.strip() for x in content_raw.split("\n")]))
    return content


def evaluate(labels: List[int], predictions: List[int]) -> Tuple[float, float, float]:
    # Calculates the same metrics included in the model evaluations (balanced accuracy and micro F1)
    # plus a standard F1
    assert len(labels) == len(predictions)
    bacc = balanced_accuracy_score(y_true=labels, y_pred=predictions)
    micro_f1 = f1_score(y_true=labels, y_pred=predictions, average="micro")
    f1 = f1_score(y_true=labels, y_pred=predictions)
    return bacc, micro_f1, f1


def main(args):
    factcc = FactCC()
    factccx = FactCCX()

    results_dict = defaultdict(dict)
    with tarfile.open(args.cnn_tar, "r") as cnn_tar:
        with tarfile.open(args.dailymail_tar, "r") as dailymail_tar:
            for split in ["val", "test"]:
                instances = read_jsonl_file(f"{args.data_dir}/{split}/data-dev.jsonl")
                inputs = []
                labels = []
                for instance in tqdm(instances):
                    candidate = instance["claim"]

                    label = 0 if instance["label"] == "CORRECT" else 1
                    labels.append(label)

                    # Remove "cnndm/"
                    filepath = instance["filepath"]
                    filepath = filepath[6:]
                    if filepath.startswith("cnn"):
                        content = cnn_tar.extractfile(f"./{filepath}").read().decode()
                    else:
                        content = (
                            dailymail_tar.extractfile(f"./{filepath}").read().decode()
                        )
                    source = parse_story_file(content)

                    inputs.append({"candidate": candidate, "sources": [source]})

                _, factcc_results = factcc.predict_batch(inputs)
                predictions = [results["factcc"]["label"] for results in factcc_results]
                acc, micro_f1, f1 = evaluate(labels, predictions)
                results_dict["factcc"][split] = {
                    "bacc": acc,
                    "micro_f1": micro_f1,
                    "f1": f1,
                }

                _, factccx_results = factccx.predict_batch(inputs)
                predictions = [
                    results["factccx"]["label"] for results in factccx_results
                ]
                acc, micro_f1, f1 = evaluate(labels, predictions)
                results_dict["factccx"][split] = {
                    "bacc": acc,
                    "micro_f1": micro_f1,
                    "f1": f1,
                }

    with open(args.output_file, "w") as out:
        out.write(json.dumps(results_dict, indent=2))


if __name__ == "__main__":
    argp = argparse.ArgumentParser()
    argp.add_argument("--cnn-tar", required=True)
    argp.add_argument("--dailymail-tar", required=True)
    argp.add_argument("--data-dir", required=True)
    argp.add_argument("--device", required=True, type=int)
    argp.add_argument("--output-file", required=True)
    args = argp.parse_args()
    main(args)



"""
This script was adapted from https://github.com/tagoyal/dae-factuality/blob/d5a472b0f18c18d7125dc1551bde59538adac765/evaluate_factuality.py
to calculate the scores for input text rather than evaluate the accuracy.
"""
# fmt: off
import argparse
import os
import json
import numpy as np
import torch
import utils
from sklearn.utils.extmath import softmax
from preprocessing_utils import get_tokens, get_relevant_deps_and_context
from transformers import (
    BertConfig,
    BertTokenizer,
    ElectraConfig,
    ElectraTokenizer,
)

MODEL_CLASSES = {"bert_dae": (BertConfig, utils.BertDAEModel, BertTokenizer),
                 "bert_basic": (BertConfig, utils.BERTBasicModel, BertTokenizer),
                 "electra_basic": (ElectraConfig, utils.ElectraBasicModel, ElectraTokenizer),
                 "electra_dae": (ElectraConfig, utils.ElectraDAEModel, ElectraTokenizer), }


def score_example_single_context(decode_text, input_text, model, tokenizer, args):
    gen_tok, _, gen_dep = get_relevant_deps_and_context(decode_text, args)

    tokenized_text = get_tokens(input_text)

    ex = {'input': tokenized_text, 'deps': [], 'context': ' '.join(gen_tok), 'sentlabel': -1}
    for dep in gen_dep:
        ex['deps'].append({'dep': dep['dep'], 'label': -1, 'head_idx': dep['head_idx'] - 1,
                           'child_idx': dep['child_idx'] - 1, 'child': dep['child'], 'head': dep['head']})

    dict_temp = {'id': 0, 'input': ex['input'], 'sentlabel': ex['sentlabel'], 'context': ex['context']}
    for i in range(20):
        if i < len(ex['deps']):
            dep = ex['deps'][i]
            dict_temp['dep_idx' + str(i)] = str(dep['child_idx']) + ' ' + str(dep['head_idx'])
            dict_temp['dep_words' + str(i)] = str(dep['child']) + ' ' + str(dep['head'])
            dict_temp['dep' + str(i)] = dep['dep']
            dict_temp['dep_label' + str(i)] = dep['label']
        else:
            dict_temp['dep_idx' + str(i)] = ''
            dict_temp['dep_words' + str(i)] = ''
            dict_temp['dep' + str(i)] = ''
            dict_temp['dep_label' + str(i)] = ''

    features = utils.convert_examples_to_features_bert(
        [dict_temp],
        tokenizer,
        max_length=128,
        pad_token=tokenizer.convert_tokens_to_ids([tokenizer.pad_token])[0],
        pad_token_segment_id=0,
    )

    # Convert to Tensors and build dataset
    input_ids = torch.tensor([f.input_ids for f in features], dtype=torch.long).to(args.device)
    attention = torch.tensor([f.input_attention_mask for f in features], dtype=torch.long).to(args.device)
    token_ids = torch.tensor([f.token_ids for f in features], dtype=torch.long).to(args.device)

    child = torch.tensor([f.child_indices for f in features], dtype=torch.long).to(args.device)
    head = torch.tensor([f.head_indices for f in features], dtype=torch.long).to(args.device)

    dep_labels = torch.tensor([f.dep_labels for f in features], dtype=torch.long).to(args.device)
    num_dependencies = torch.tensor([f.num_dependencies for f in features], dtype=torch.long).to(args.device)
    arcs = torch.tensor([f.arcs for f in features], dtype=torch.long).to(args.device)
    arc_labels = torch.tensor([f.arc_labels for f in features], dtype=torch.long).to(args.device)
    arc_label_lengths = torch.tensor([f.arc_label_lengths for f in features], dtype=torch.long).to(args.device)

    inputs = {'input_ids': input_ids, 'attention': attention, 'token_ids': token_ids, 'child': child, 'head': head,
              'dep_labels': dep_labels, 'arcs': arc_labels, 'arc_label_lengths': arc_label_lengths,
              'device': args.device}

    outputs = model(**inputs)
    tmp_eval_loss, logits = outputs[:2]
    preds = logits.detach().cpu().numpy()

    f_out = open('test.txt', 'a')
    text = tokenizer.decode(input_ids[0])
    text = text.replace(tokenizer.pad_token, '').strip()
    f_out.write(text + '\n')
    for j, arc in enumerate(arcs[0]):
        arc_text = tokenizer.decode(arc)
        arc_text = arc_text.replace(tokenizer.pad_token, '').strip()

        if arc_text == '':
            break

        pred_temp = softmax([preds[0][j]])
        f_out.write(arc_text + '\n')
        f_out.write('pred:\t' + str(np.argmax(pred_temp)) + '\n')
        f_out.write(str(pred_temp[0][0]) + '\t' + str(pred_temp[0][1]) + '\n')
        f_out.write('\n')

    f_out.close()
    preds = preds.reshape(-1, 2)
    preds = softmax(preds)
    preds = preds[:, 1]
    preds = preds[:num_dependencies[0]]
    score = np.mean(preds)
    return score


def main():
    parser = argparse.ArgumentParser()

    # Required parameters
    parser.add_argument(
        "--model_type",
        default=None,
        type=str,
        required=True,
        help="Model type selected in the list: " + ", ".join(MODEL_CLASSES.keys()),
    )
    parser.add_argument(
        "--input_dir",
        default=None,
        type=str,
        required=True,
        help="The output directory where the model predictions and checkpoints will be written.",
    )

    # Other parameters
    parser.add_argument(
        "--per_gpu_eval_batch_size", default=8, type=int, help="Batch size per GPU/CPU for evaluation.",
    )
    parser.add_argument(
        "--max_seq_length",
        default=128,
        type=int,
        help="The maximum total input sequence length after tokenization. Sequences longer "
             "than this will be truncated, sequences shorter will be padded.",
    )
    parser.add_argument("--include_sentence_level", action="store_true", help="Overwrite the cached data sets", )
    parser.add_argument("--gpu_device", type=int, default=0, help="gpu device")
    parser.add_argument("--dependency_type", default='enhancedDependencies', help='type of dependency labels')

    parser.add_argument("--input-file", required=True)
    parser.add_argument("--output-file", required=True)

    args = parser.parse_args()

    args.n_gpu = 1
    if args.gpu_device != -1:
        device = torch.device("cuda", args.gpu_device)
    else:
        device = torch.device("cpu")
    args.device = device

    print(args.input_dir)
    if not os.path.exists(args.input_dir):
        print('Check model location')

    args.model_type = args.model_type.lower()
    config_class, model_class, tokenizer_class = MODEL_CLASSES[args.model_type]

    tokenizer = tokenizer_class.from_pretrained(args.input_dir)
    model = model_class.from_pretrained(args.input_dir)
    model.to(args.device)

    with open(args.output_file, "w") as out:
        with open(args.input_file, "r") as f:
            for line in f:
                data = json.loads(line)
                candidate = data["candidate"]
                source = data["source"]

                score = score_example_single_context(candidate, source, model, tokenizer, args)
                out.write(json.dumps({"dae": float(score)}) + "\n")


if __name__ == "__main__":
    main()



import json
import unittest
from parameterized import parameterized

from repro.models.goyal2020 import DAE
from repro.testing import FIXTURES_ROOT as REPRO_FIXTURES_ROOT
from repro.testing import assert_dicts_approx_equal, get_testing_device_parameters

from . import FIXTURES_ROOT


def _get_testing_parameters():
    devices = get_testing_device_parameters()
    # `None` will be used to indicate the default model
    models = [None, "dae_basic", "dae_w_syn", "dae_w_syn_hallu"]

    parameters = []
    for (device,) in devices:
        for model in models:
            parameters.append((device, model))
    return parameters


class TestGoyal2020Models(unittest.TestCase):
    def setUp(self) -> None:
        self.expected = json.load(open(f"{FIXTURES_ROOT}/expected.json", "r"))
        self.examples = json.load(
            open(f"{REPRO_FIXTURES_ROOT}/multiling2011/data.json", "r")
        )

    @parameterized.expand(_get_testing_parameters())
    def test_dae_regression(self, device: int, model_name: str):
        if model_name is None:
            # Use the default model, which is "dae_w_syn" and
            # set the model name for usage later on
            model = DAE(device=device)
            model_name = "dae_w_syn"
        else:
            model = DAE(model=model_name, device=device)

        # The inputs to DAE are single sentences, so we take the
        # first sentence of the candidate and first sentence of the
        # first source document as input
        inputs = [
            {
                "candidate": example["candidate"][0],
                "sources": [example["sources"][0][0]],
            }
            for example in self.examples
        ]
        expected_macro = self.expected[model_name]["macro"]
        expected_micro = self.expected[model_name]["micro"]
        actual_macro, actual_micro = model.predict_batch(inputs)

        assert_dicts_approx_equal(expected_macro, actual_macro, abs=1e-4)
        assert len(expected_micro) == len(actual_micro)
        for expected, actual in zip(expected_micro, actual_micro):
            assert_dicts_approx_equal(expected, actual, abs=1e-4)



import unittest
from parameterized import parameterized

from repro.models.thompson2020 import Prism
from repro.testing import assert_dicts_approx_equal, get_testing_device_parameters


class TestThompson202Models(unittest.TestCase):
    @parameterized.expand(get_testing_device_parameters())
    def test_prism_references_example(self, device: int):
        # Tests the examples from the Github repo
        model = Prism(device=device)
        inputs = [
            {"candidate": "Hi world.", "references": ["Hello world."]},
            {"candidate": "This is a Test.", "references": ["This is a test."]},
        ]
        expected_macro = {"prism": -1.0184666}
        expected_micro = [{"prism": -1.4878583}, {"prism": -0.5490748}]
        actual_macro, actual_micro = model.predict_batch(inputs)

        assert_dicts_approx_equal(expected_macro, actual_macro)
        assert len(expected_micro) == len(actual_micro)
        for expected, actual in zip(expected_micro, actual_micro):
            assert_dicts_approx_equal(expected, actual)

    @parameterized.expand(get_testing_device_parameters())
    def test_prism_sources_example(self, device: int):
        # Tests the examples from the Github repo
        model = Prism(device=device)
        inputs = [
            {"candidate": "Hi world.", "sources": ["Bonjour le monde."]},
            {"candidate": "This is a Test.", "sources": ["C'est un test."]},
        ]
        expected_macro = {"prism": -1.8306842}
        expected_micro = [{"prism": -2.462842}, {"prism": -1.1985264}]
        actual_macro, actual_micro = model.predict_batch(inputs)

        assert_dicts_approx_equal(expected_macro, actual_macro)
        assert len(expected_micro) == len(actual_micro)
        for expected, actual in zip(expected_micro, actual_micro):
            assert_dicts_approx_equal(expected, actual)

    def test_invalid_input(self):
        model = Prism()
        inputs = [{"candidate": "Hi world."}, {"candidate": "This is a Test."}]
        with self.assertRaises(Exception):
            model.predict_batch(inputs)

        inputs = [
            {"candidate": "Hi world.", "references": ["Hello world."]},
            {"candidate": "This is a Test.", "sources": ["C'est un test."]},
        ]
        with self.assertRaises(Exception):
            model.predict_batch(inputs)

        inputs = [
            {"candidate": "Hi world.", "references": ["Hello world."]},
            {
                "candidate": "This is a Test.",
            },
        ]
        with self.assertRaises(Exception):
            model.predict_batch(inputs)

        inputs = [
            {
                "candidate": "Hi world.",
            },
            {"candidate": "This is a Test.", "sources": ["C'est un test."]},
        ]
        with self.assertRaises(Exception):
            model.predict_batch(inputs)



import os

VERSION = "1.0"
MODEL_NAME = os.path.basename(os.path.dirname(__file__))
DOCKERHUB_REPRO = f"danieldeutsch/{MODEL_NAME}"
DEFAULT_IMAGE = f"{DOCKERHUB_REPRO}:{VERSION}"
AUTOMATICALLY_PUBLISH = True

from repro.models.chen2020.dataset_reader import Chen2020EvaluationDatasetReader
from repro.models.chen2020.models import LERC, MOCHAEvaluationMetric
from repro.models.chen2020.setup import Chen2020SetupSubcommand



import json
import unittest
from parameterized import parameterized

from repro.models.kryscinski2019 import FactCC, FactCCX
from repro.testing import FIXTURES_ROOT as REPRO_FIXTURES_ROOT
from repro.testing import assert_dicts_approx_equal, get_testing_device_parameters

from . import FIXTURES_ROOT


class TestKryscinski2019Models(unittest.TestCase):
    def setUp(self) -> None:
        self.expected = json.load(open(f"{FIXTURES_ROOT}/expected.json", "r"))
        self.examples = json.load(
            open(f"{REPRO_FIXTURES_ROOT}/multiling2011/data.json", "r")
        )

    @parameterized.expand(get_testing_device_parameters())
    def test_factcc_regression(self, device: int):
        model = FactCC(device=device)
        inputs = [
            {
                "candidate": example["candidate"],
                "sources": [example["sources"][0]],
            }
            for example in self.examples
        ]
        expected_macro = self.expected["factcc"]["macro"]
        expected_micro = self.expected["factcc"]["micro"]
        actual_macro, actual_micro = model.predict_batch(inputs)

        assert_dicts_approx_equal(expected_macro, actual_macro, abs=1e-4)
        assert len(expected_micro) == len(actual_micro)
        for expected, actual in zip(expected_micro, actual_micro):
            assert_dicts_approx_equal(expected, actual, abs=1e-4)

    @parameterized.expand(get_testing_device_parameters())
    def test_factccx_regression(self, device: int):
        model = FactCCX(device=device)
        inputs = [
            {
                "candidate": example["candidate"],
                "sources": [example["sources"][0]],
            }
            for example in self.examples
        ]
        expected_macro = self.expected["factccx"]["macro"]
        expected_micro = self.expected["factccx"]["micro"]
        actual_macro, actual_micro = model.predict_batch(inputs)

        assert_dicts_approx_equal(expected_macro, actual_macro, abs=1e-4)
        assert len(expected_micro) == len(actual_micro)
        for expected, actual in zip(expected_micro, actual_micro):
            assert_dicts_approx_equal(expected, actual, abs=1e-4)



import json
import logging
import numpy as np
from typing import Dict, List, Tuple, Union

from repro.common import util
from repro.common.docker import DockerContainer
from repro.data.types import MetricsType, TextType
from repro.models import Model
from repro.models.sellam2020 import DEFAULT_IMAGE, MODEL_NAME

logger = logging.getLogger(__name__)


@Model.register(f"{MODEL_NAME}-bleurt")
class BLEURT(Model):
    def __init__(
        self,
        image: str = DEFAULT_IMAGE,
        model: str = "bleurt-base-128",
        device: int = 0,
        batch_size: int = 16,
    ):
        self.image = image
        self.model = model
        self.device = device
        self.batch_size = batch_size

    def predict(
        self,
        candidate: TextType,
        references: List[TextType],
        **kwargs,
    ) -> MetricsType:
        return self.predict_batch(
            [{"candidate": candidate, "references": references}], **kwargs
        )[0]

    def predict_batch(
        self, inputs: List[Dict[str, Union[TextType, List[TextType]]]], **kwargs
    ) -> Tuple[MetricsType, List[MetricsType]]:
        logger.info(
            f"Calculating BLEURT with model {self.model} and "
            f"image {self.image} on {len(inputs)} inputs."
        )

        candidates = [inp["candidate"] for inp in inputs]
        references_list = [inp["references"] for inp in inputs]

        # The each candidate and reference must be `str`, not `List[str]`
        candidates = [util.flatten(candidate) for candidate in candidates]
        references_list = [
            [util.flatten(reference) for reference in references]
            for references in references_list
        ]

        with DockerContainer(self.image) as backend:
            host_input_file = f"{backend.host_dir}/input.jsonl"
            container_input_file = f"{backend.container_dir}/input.jsonl"

            # BLEURT only runs with a single reference, so we write
            # the candidate with each of its references on its own. Later
            # the scores will be aggregated.
            with open(host_input_file, "w") as out:
                for candidate, references in zip(candidates, references_list):
                    for reference in references:
                        out.write(
                            json.dumps(
                                {
                                    "candidate": candidate,
                                    "reference": reference,
                                }
                            )
                            + "\n"
                        )

            host_output_file = f"{backend.host_dir}/output.jsonl"
            container_output_file = f"{backend.container_dir}/output.jsonl"

            cuda = self.device != -1
            commands = []
            if cuda:
                commands.append(f"export CUDA_VISIBLE_DEVICES={self.device}")
            commands.append("cd bleurt")
            commands.append(
                f"python -m bleurt.score_files"
                f"  -sentence_pairs_file {container_input_file}"
                f"  -bleurt_checkpoint ../{self.model}"
                f"  -scores_file {container_output_file}"
                f"  -bleurt_batch_size {self.batch_size}"
            )

            command = " && ".join(commands)
            backend.run_command(
                command=command,
                cuda=cuda,
                network_disabled=True,
            )

            results = open(host_output_file, "r").read().splitlines()
            results = list(map(float, results))

            # Regroup by reference
            micro_metrics = []
            index = 0
            for references in references_list:
                scores = results[index : index + len(references)]
                index += len(references)
                micro_metrics.append(
                    {"bleurt": {"mean": np.mean(scores), "max": np.max(scores)}}
                )

            macro_metrics = util.average_dicts(micro_metrics)
            return macro_metrics, micro_metrics



import json
import logging
from typing import Dict, List, Tuple, Union

from repro.common import util
from repro.common.docker import DockerContainer
from repro.common.io import read_jsonl_file
from repro.data.types import MetricsType, TextType
from repro.models import Model
from repro.models.thompson2020 import DEFAULT_IMAGE, MODEL_NAME

logger = logging.getLogger(__name__)


@Model.register(f"{MODEL_NAME}-prism")
class Prism(Model):
    def __init__(
        self, image: str = DEFAULT_IMAGE, device: int = 0, language: str = "en"
    ):
        self.image = image
        self.device = device
        self.language = language

    @staticmethod
    def _check_single_text(texts_list: List[List[TextType]]) -> List[TextType]:
        single_texts = []
        for texts in texts_list:
            if texts is None:
                single_texts.append(None)
            else:
                if len(texts) != 1:
                    raise Exception(
                        f"Prism only supports single sources and references. Found: {len(texts)}"
                    )
                single_texts.append(texts[0])
        return single_texts

    def predict(
        self,
        candidate: TextType,
        sources: List[TextType] = None,
        references: List[TextType] = None,
        **kwargs,
    ) -> MetricsType:
        return self.predict_batch(
            [{"candidate": candidate, "sources": sources, "references": references}],
            **kwargs,
        )[0]

    def predict_batch(
        self, inputs: List[Dict[str, Union[TextType, List[TextType]]]], **kwargs
    ) -> Tuple[MetricsType, List[MetricsType]]:
        logger.info(f"Calculating Prism for {len(inputs)} inputs")

        candidates = [inp["candidate"] for inp in inputs]
        sources_list = [inp["sources"] if "sources" in inp else None for inp in inputs]
        references_list = [
            inp["references"] if "references" in inp else None for inp in inputs
        ]

        # Prism only supports single sources and references
        sources = self._check_single_text(sources_list)
        references = self._check_single_text(references_list)

        # Ensure all are strings or None
        candidates = [util.flatten(candidate) for candidate in candidates]
        sources = [
            util.flatten(source) if source is not None else None for source in sources
        ]
        references = [
            util.flatten(reference) if reference is not None else None
            for reference in references
        ]

        # Prism only supports having references xor sources. Ensure this is true
        has_references = all(reference is not None for reference in references)
        has_sources = all(source is not None for source in sources)
        if has_references and has_sources or (not has_references and not has_sources):
            raise Exception(
                f"Prism supports having either input references xor sources, not both or neither."
            )

        with DockerContainer(self.image) as backend:
            host_input_file = f"{backend.host_dir}/input.jsonl"
            container_input_file = f"{backend.container_dir}/input.jsonl"
            with open(host_input_file, "w") as out:
                for candidate, source, reference in zip(
                    candidates, sources, references
                ):
                    out.write(
                        json.dumps(
                            {
                                "candidate": candidate,
                                "source": source,
                                "reference": reference,
                            }
                        )
                        + "\n"
                    )

            host_output_file = f"{backend.host_dir}/output.jsonl"
            container_output_file = f"{backend.container_dir}/output.jsonl"

            cuda = self.device != -1
            commands = []
            if cuda:
                commands.append(f"export CUDA_VISIBLE_DEVICES={self.device}")

            commands.append("cd prism")
            commands.append(
                f"python score.py"
                f"  --input-file {container_input_file}"
                f"  --language {self.language}"
                f"  --output-file {container_output_file}"
            )

            command = " && ".join(commands)
            backend.run_command(command=command, cuda=cuda, network_disabled=True)
            micro_metrics = read_jsonl_file(host_output_file)
            macro_metrics = util.average_dicts(micro_metrics)
            return macro_metrics, micro_metrics



import argparse
import json

from summaqa import QA_Metric, QG_masked


def main(args):
    question_generator = QG_masked()
    qa_metric = QA_Metric()

    with open(args.output_file, "w") as out:
        with open(args.input_file, "r") as f:
            for line in f:
                data = json.loads(line)
                candidate = data["candidate"]
                source = data["source"]

                masked_questions, answer_spans = question_generator.get_questions(
                    source
                )
                score = qa_metric.compute(masked_questions, answer_spans, candidate)
                out.write(json.dumps({"summaqa": score}) + "\n")


if __name__ == "__main__":
    argp = argparse.ArgumentParser()
    argp.add_argument("--input-file", required=True)
    argp.add_argument("--output-file", required=True)
    args = argp.parse_args()
    main(args)



import os

VERSION = "1.0"
MODEL_NAME = os.path.basename(os.path.dirname(__file__))
DOCKERHUB_REPRO = f"danieldeutsch/{MODEL_NAME}"
DEFAULT_IMAGE = f"{DOCKERHUB_REPRO}:{VERSION}"
AUTOMATICALLY_PUBLISH = True

from repro.models.liu2019.models import BertSumExt, BertSumExtAbs, TransformerAbs
from repro.models.liu2019.setup import Liu2019SetupSubcommand



import os

VERSION = "1.0"
MODEL_NAME = os.path.basename(os.path.dirname(__file__))
DOCKERHUB_REPRO = f"danieldeutsch/{MODEL_NAME}"
DEFAULT_IMAGE = f"{DOCKERHUB_REPRO}:{VERSION}"
AUTOMATICALLY_PUBLISH = True

from repro.models.scialom2021.models import (
    QuestEval,
    QuestEvalForSimplification,
    QuestEvalForSummarization,
)
from repro.models.scialom2021.setup import Scialom2021SetupSubcommand



import argparse
import json

from repro.models.goyal2020 import DAE


def main(args):
    data = json.load(open(args.input_file, "r"))
    correct_inputs = []
    incorrect_inputs = []
    for instance in data:
        correct_inputs.append(
            {
                "candidate": instance["correct_sent"],
                "sources": [instance["article_sent"]],
            }
        )
        incorrect_inputs.append(
            {
                "candidate": instance["incorrect_sent"],
                "sources": [instance["article_sent"]],
            }
        )

    results = {}
    for name in ["dae_basic", "dae_w_syn", "dae_w_syn_hallu"]:
        model = DAE(model=name, device=args.device)
        _, correct_scores = model.predict_batch(correct_inputs)
        _, incorrect_scores = model.predict_batch(incorrect_inputs)
        total = len(correct_scores)
        num_correct = 0
        for correct, incorrect in zip(correct_scores, incorrect_scores):
            if correct["dae"] > incorrect["dae"]:
                num_correct += 1
        accuracy = num_correct / total
        results[name] = accuracy

    with open(args.output_file, "w") as out:
        out.write(json.dumps(results, indent=2))


if __name__ == "__main__":
    argp = argparse.ArgumentParser()
    argp.add_argument("--input-file", required=True)
    argp.add_argument("--device", required=True, type=int)
    argp.add_argument("--output-file", required=True)
    args = argp.parse_args()
    main(args)



import json
import unittest

from repro.models.scialom2019 import SummaQA
from repro.testing import FIXTURES_ROOT as REPRO_FIXTURES_ROOT
from repro.testing import assert_dicts_approx_equal

from . import FIXTURES_ROOT


class TestScialom2019Models(unittest.TestCase):
    def setUp(self) -> None:
        self.examples = json.load(
            open(f"{REPRO_FIXTURES_ROOT}/multiling2011/data.json", "r")
        )
        self.expected = json.load(open(f"{FIXTURES_ROOT}/expected.json", "r"))
        self.summaqa_examples = json.load(
            open(f"{FIXTURES_ROOT}/summaqa-examples.json", "r")
        )

    def test_summaqa_examples(self):
        # Tests the examples provided in the Github repo
        model = SummaQA()
        inputs = self.summaqa_examples["inputs"]
        expected_micro = self.summaqa_examples["outputs"]
        _, actual_micro = model.predict_batch(inputs)

        assert len(expected_micro) == len(actual_micro)
        for expected, actual in zip(expected_micro, actual_micro):
            assert_dicts_approx_equal(expected, actual, abs=1e-4)

    def test_summaqa_regression(self):
        model = SummaQA()
        # We only take the first source since SummaQA only supports one of each
        inputs = [
            {
                "candidate": inp["candidate"],
                "sources": [inp["sources"][0]],
            }
            for inp in self.examples
        ]
        expected_macro = self.expected["macro"]
        expected_micro = self.expected["micro"]
        actual_macro, actual_micro = model.predict_batch(inputs)

        assert_dicts_approx_equal(expected_macro, actual_macro, abs=1e-4)
        assert len(expected_micro) == len(actual_micro)
        for expected, actual in zip(expected_micro, actual_micro):
            assert_dicts_approx_equal(expected, actual, abs=1e-4)



import argparse
import json

from prism import Prism


def main(args):
    metric = Prism(model_dir="../m39v1", lang=args.language)

    candidates = []
    references = []
    sources = []

    with open(args.input_file, "r") as f:
        for line in f:
            data = json.loads(line)
            candidates.append(data["candidate"])
            references.append(data["reference"])
            sources.append(data["source"])

    # Input error checking is done in the model code, so we don't need to check here.
    # Assume that if there are references, they all have references and no sources.
    has_references = any(reference is not None for reference in references)
    if has_references:
        scores = metric.score(cand=candidates, ref=references, segment_scores=True)
    else:
        # Assume all have sources
        scores = metric.score(cand=candidates, src=sources, segment_scores=True)

    with open(args.output_file, "w") as out:
        for score in scores:
            out.write(json.dumps({"prism": float(score)}) + "\n")


if __name__ == "__main__":
    argp = argparse.ArgumentParser()
    argp.add_argument("--input-file", required=True)
    argp.add_argument("--language", required=True)
    argp.add_argument("--output-file", required=True)
    args = argp.parse_args()
    main(args)



import json
import logging
from typing import Dict, List, Tuple, Union

from repro.common import util
from repro.common.docker import DockerContainer
from repro.common.io import read_jsonl_file
from repro.data.types import MetricsType, TextType
from repro.models import Model
from repro.models.scialom2019 import DEFAULT_IMAGE, MODEL_NAME

logger = logging.getLogger(__name__)


@Model.register(f"{MODEL_NAME}-summaqa")
class SummaQA(Model):
    def __init__(self, image: str = DEFAULT_IMAGE):
        self.image = image

    def predict(
        self,
        candidate: TextType,
        sources: List[TextType],
        **kwargs,
    ) -> MetricsType:
        return self.predict_batch(
            [{"candidate": candidate, "sources": sources}],
            **kwargs,
        )[0]

    def predict_batch(
        self, inputs: List[Dict[str, Union[TextType, List[TextType]]]], **kwargs
    ) -> Tuple[MetricsType, List[MetricsType]]:
        logger.info(f"Calculating SummaQA for {len(inputs)} inputs")

        candidates = [inp["candidate"] for inp in inputs]
        sources_list = [inp["sources"] for inp in inputs]

        # SummaQA only supports single sources
        sources = util.check_for_single_texts(sources_list)

        # Ensure all are strings
        candidates = [util.flatten(candidate) for candidate in candidates]
        sources = [util.flatten(source) for source in sources]

        with DockerContainer(self.image) as backend:
            host_input_file = f"{backend.host_dir}/input.jsonl"
            container_input_file = f"{backend.container_dir}/input.jsonl"
            with open(host_input_file, "w") as out:
                for candidate, source in zip(candidates, sources):
                    out.write(
                        json.dumps(
                            {
                                "candidate": candidate,
                                "source": source,
                            }
                        )
                        + "\n"
                    )

            host_output_file = f"{backend.host_dir}/output.jsonl"
            container_output_file = f"{backend.container_dir}/output.jsonl"

            command = (
                f"python score.py"
                f"  --input-file {container_input_file}"
                f"  --output-file {container_output_file}"
            )

            backend.run_command(command=command, network_disabled=True)
            micro_metrics = read_jsonl_file(host_output_file)
            macro_metrics = util.average_dicts(micro_metrics)
            return macro_metrics, micro_metrics
